@@ -18,23 +18,21 @@ use failures qw{
 use Hash::Ordered;
 use PDL::Basic qw(sequence);
 use PDL::Core qw(pdl null);
-use List::AllUtils 0.19 qw(
-  each_arrayref pairgrep pairkeys pairmap pairwise reduce zip
-);
-
-use PDL::DateTime  ();
+use Data::Perl     ();
+use Data::Perl::Collection::Array;
+use List::AllUtils qw(each_arrayref pairgrep pairkeys pairmap pairwise);
+use List::MoreUtils 0.423;
 use PDL::Primitive ();
 use PDL::Factor    ();
 use PDL::SV        ();
 use PDL::StringfiableExtension;
 use Ref::Util qw(is_plain_arrayref is_plain_hashref);
 use Scalar::Util qw(blessed looks_like_number);
+use Sereal::Decoder 4.005;
 use Sereal::Encoder 4.005;
 use Text::Table::Tiny;
 use Type::Params;
-use Types::Standard qw(
-  Any ArrayRef CodeRef Enum CycleTuple HashRef Maybe Str
-);
+use Types::Standard qw(Any ArrayRef CodeRef CycleTuple HashRef Maybe Str);
 use Types::PDL qw(Piddle Piddle1D);
 
 use Data::Frame::Column::Helper;
@@ -80,6 +78,13 @@ use overload (
     fallback => 1
 );
 
+{
+    # TODO temporary column role
+    no strict 'refs';
+    *{'PDL::number_of_rows'} = sub { $_[0]->getdim(0) };
+    *{'Data::Perl::Collection::Array::number_of_rows'} = sub { $_[0]->count };
+}
+
 # Relative tolerance. This can be used for data frame comparison.
 our $TOLERANCE_REL = undef;
 our $doubleformat = '%.8g';
@@ -159,38 +164,38 @@ sub BUILD {
 }
 
 
-method _format_cell ($col, $ridx,
-                     :$na='BAD', :$isbad=$col->isbad)
-{
-    my $is_numeric = $self->_is_numeric_column($col);
-    if ($isbad->at($ridx)) {
-        return $is_numeric ? $na : "<$na>";
-    }
-
-    if ( $col->$_DOES('PDL::DateTime') ) {
-        return $col->dt_at($ridx);
-    }
-    elsif ( $is_numeric ) {
-        if ( $col->type >= PDL::float ) {
-
-            # This is to fix some float precision problem with perl
-            # of nvsize=16, which can cause $df->string to get untidy
-            # float data to cause our test to fail.
-            return
-              sprintf( $doubleformat, $col->slice($ridx)->squeeze->string );
-        }
-    }
-    return $col->slice($ridx)->squeeze->string;
-}
-
 method _string() {
+    my $format_cell = sub {
+        my ( $col, $ridx ) = @_;
+
+        if ( $col->$_DOES('PDL::DateTime') ) {
+            return $col->dt_at($ridx);
+        }
+        elsif ( $self->_is_numeric_column($col) ) {
+            if ( $col->type >= PDL::float ) {
+
+                # This is to fix some float precision problem with perl
+                # of nvsize=16, which can cause $df->string to get untidy
+                # float data to cause our test to fail.
+                my $s = $col->slice($ridx)->squeeze->string;
+                if ( $s eq 'BAD' ) {
+                    return $s;
+                }
+                else {
+                    return sprintf( $doubleformat, $s );
+                }
+            }
+        }
+        return $col->slice($ridx)->squeeze->string;
+    };
+
     my @rows = ( [ '', @{ $self->column_names } ] );
     for my $r_idx ( 0 .. $self->number_of_rows - 1 ) {
         my $r = [
             $self->row_names->slice($r_idx)->squeeze->string,
             map {
                 my $col = $self->nth_column($_);
-                $self->_format_cell( $col, $r_idx );
+                $format_cell->( $col, $r_idx );
             } 0 .. $self->number_of_columns - 1
         ];
         push @rows, $r;
@@ -301,6 +306,9 @@ method at (@rest) {
 }
 
 
+# Public methods other than slice() shall be non-lvalue.
+sub select_columns { shift->_select_columns(@_); }
+
 method exists ($col_name) {
     $self->_columns->exists($col_name);
 }
@@ -356,6 +364,21 @@ method set ($indexer, $data) {
 
 method isempty () { $self->nrow == 0; }
 
+# supports negative indices
+method nth_column($index) {
+	failure::index->throw({
+			msg => "requires index",
+			trace => failure->croak_trace
+		}) unless defined $index;
+	failure::index::exists->throw({
+			msg => "column index out of bounds",
+			trace => failure->croak_trace,
+		}) if $index >= $self->number_of_columns;
+	# fine if $index < 0 because negative indices are supported
+	return ($self->_columns->values)[$index];
+}
+
+
 
 method column_names(@rest) {
     my @colnames =
@@ -377,10 +400,7 @@ method column_names(@rest) {
         # rename column names
         my @values = $self->_columns->values;
         $self->_columns->clear;
-
-        # List::AllUtils and List::Util 's zip func are different.
-        # See also https://github.com/houseabsolute/List-AllUtils/issues/12
-        $self->_columns->push( zip( @colnames, @values ) );
+        $self->_columns->push( List::AllUtils::zip( @colnames, @values ) );
 	}
 	return [ $self->_columns->keys ];
 }
@@ -394,25 +414,30 @@ sub row_names {
 		# setting row names
 		my $new_rows;
         if ( ref $rest[0] ) {
-            if ( Ref::Util::is_plain_arrayref($rest[0]) ) {
-                $new_rows = [ @{$rest[0]} ];
+            if ( ref $rest[0] eq 'ARRAY' ) {
+                $new_rows = Data::Perl::array( @{ $rest[0] } );
             }
             elsif ( $rest[0]->isa('PDL') ) {
 
                 # TODO just run uniq?
-                $new_rows = $rest[0]->unpdl;
+                $new_rows = Data::Perl::array( @{ $rest[0]->unpdl } );
+            }
+            else {
+                $new_rows = Data::Perl::array(@rest);
             }
         }
-        $new_rows //= [ @rest ];
+        else {
+            $new_rows = Data::Perl::array(@rest);
+        }
 
 		failure::rows::length->throw({
 				msg => "invalid row names length",
 				trace => failure->croak_trace,
-			}) if $self->number_of_rows != $new_rows->length;
+			}) if $self->number_of_rows != $new_rows->number_of_rows;
 		failure::rows::unique->throw({
 				msg => "non-unique row names",
 				trace => failure->croak_trace,
-			}) if $new_rows->length != $new_rows->uniq->length;
+			}) if $new_rows->number_of_rows != $new_rows->uniq->number_of_rows;
 
 		return $self->_row_names( PDL::SV->new($new_rows) );
 	}
@@ -437,24 +462,9 @@ method column($colname) {
 			msg => "column $colname does not exist",
 			trace => failure->croak_trace,
 		}) unless $self->exists( $colname );
-	return $self->_columns->get($colname);
+	return $self->_columns->get( $colname );
 }
 
-# supports negative indices
-method nth_column($index) {
-	failure::index->throw({
-			msg => "requires index",
-			trace => failure->croak_trace
-		}) unless defined $index;
-	failure::index::exists->throw({
-			msg => "column index out of bounds",
-			trace => failure->croak_trace,
-		}) if $index >= $self->number_of_columns;
-	# fine if $index < 0 because negative indices are supported
-	return ($self->_columns->values)[$index];
-}
-
-
 sub _column_validate {
 	my ($self, $name, $data) = @_;
 	if( $name =~ /^\d+$/  ) {
@@ -501,88 +511,6 @@ sub add_column {
 	$self->_columns->push( $name => $data );
 }
 
-
-method copy () { 
-    return ref($self)->new(
-        columns   => $self->names->map( sub { $_ => $self->column($_)->copy } ),
-        row_names => $self->row_names->copy
-    );
-}
-*clone = \&copy;
-
-
-method summary ($percentiles=[0.25, 0.75]) {
-    if ( List::AllUtils::any { $_ < 0 or $_ > 1 } @$percentiles ) {
-        die "percentiles should all be in the interval [0, 1].";
-    }
-
-    state $std = sub {
-        my ($p) = @_;
-        $p = $p->where( $p->isgood );
-        return sqrt( ( ( ( $p - $p->average )**2 )->sum ) / $p->length );
-    };
-
-    my $class = ref($self);
-    my @pct   = sort { $a <=> $b }
-      List::AllUtils::uniq( ( ( $percentiles->flatten ), 0.5 ) );
-    my %summary = map {
-        my $col = $self->column($_);
-
-        my $count = $col->ngood;
-        if ( $self->is_numeric_column($_) ) {
-            my $average  = $col->average;
-            my $min      = $col->min;
-            my $max      = $col->max;
-            my @pct_data = map { $col->pct($_) } @pct;
-            if ( $col->$_DOES('PDL::DateTime') ) {
-                $_ => PDL::DateTime->new(
-                    [ 0, $average, 0, $min, @pct_data, $max ] )
-                  ->setbadif( pdl( [ 1, 0, 1, 0, ( (0) x @pct ), 0 ] ) );
-            }
-            else {
-                $_ => pdl(
-                    [
-                        $count, $average,  $std->($col),
-                        $min,   @pct_data, $max
-                    ]
-                );
-            }
-        }
-        else {
-            $_ =>
-              pdl( [ $count, ( ("nan") x ( @pct + 4 ) ) ] )->setnantobad();
-        }
-    } $self->names->flatten;
-    return $class->new(
-        columns   => [ map { $_ => $summary{$_} } $self->names->flatten ],
-        row_names => [
-            qw(count mean std min),
-            (
-                map {
-                    my $nof_digits = int( 4 + log($_) / log(10) );
-                    sprintf( "%.${nof_digits}g%%", $_ * 100 );
-                } @pct
-            ),
-            'max'
-        ],
-    );
-}
-
-
-method select_columns (@rest) {
-    my $indexer = indexer_s(@rest);
-    return $self if ( not defined $indexer or $indexer->indexer->length == 0 );
-
-    my $indices      = $self->_cindexer_to_indices($indexer);
-    my $column_names = $self->column_names;
-    return ref($self)->new(
-        columns => $indices->map(
-            sub { $column_names->at($_) => $self->_nth_column($_) }
-        ),
-        row_names => $self->row_names
-    );
-}
-
 # R
 # > iris[c(1,2,3,3,3,3),]
 # PDL
@@ -609,10 +537,137 @@ method select_rows(@rest) {
 }
 
 
+method sample ($n) {
+    if ($n > $self->nrow) {
+        die "sample size is larger than nrow";
+    }
+
+    my $indices = [ List::MoreUtils::samples($n, (0 .. $self->nrow-1)) ];
+    return $self->select_rows($indices);
+}
+
+
+method merge (DataFrame $df) {
+    my $class   = ref($self);
+    my $columns = [
+        $self->names->map( sub { $_ => $self->at($_) } )->flatten,
+        $df->names->map( sub { $_ => $df->at($_) } )->flatten
+    ];
+    return $class->new(
+        columns   => $columns,
+        row_names => $self->row_names
+    );
+}
+*cbind = \&merge;
+
+method append (DataFrame $df) {
+    if ( $df->nrow == 0 ) {                     # $df is empty
+        return $self->clone();
+    }
+    if ( $self->column_names->length == 0) {    # $self has no columns
+        return $df->clone;
+    }
+
+    my $class   = ref($self);
+    my $columns = $self->names->map(
+        sub {
+            my $col = $self->at($_);
+            # use glue() as PDL's append() cannot handle bad values
+            $_ => $col->glue( 0, $df->at($_) );
+        }
+    );
+    return $class->new( columns => $columns );
+}
+*rbind = \&append;
+
+method transform ($func) {
+    state $check = Type::Params::compile(
+        (
+            CodeRef | ( HashRef [ Maybe [CodeRef] ] ) |
+              ( CycleTuple [ Str, Maybe [CodeRef] ] )
+        )
+    );
+    ($func) = $check->($func);
+
+    my $class = ref($self);
+
+    my @columns;
+    if ( Ref::Util::is_coderef($func) ) {
+        @columns =
+          $self->names->map( sub {
+            $_ => $func->( $self->at($_), $self );
+          } )->flatten;
+    }
+    else {    # hashref or arrayref
+        my $column_names = $self->names;
+        my $hashref;
+        my @new_column_names;
+        if ( Ref::Util::is_hashref($func) ) {
+            $hashref = $func;
+            @new_column_names =
+              grep { !$self->exists($_) } sort( keys %$hashref );
+        }
+        else {    # arrayref
+            $hashref = {@$func};
+            @new_column_names = grep { !$self->exists($_) } ( pairkeys @$func );
+        }
+
+        @columns = $column_names->map(
+            sub {
+                my $f = exists($hashref->{$_}) ? $hashref->{$_} : sub { $_[0] };
+                $f //= sub { undef };
+                $_ => $f->( $self->at($_), $self );
+            }
+        )->flatten;
+        push @columns,
+          map { my $f = $hashref->{$_}; $_ => $f->( undef, $self ) }
+          @new_column_names;
+    }
+
+    my %columns_to_drop = @columns;
+    %columns_to_drop = pairgrep { not defined $b } %columns_to_drop;
+
+    return $class->new(
+        columns   => [ pairgrep { !exists($columns_to_drop{$a}) } @columns ],
+        row_names => $self->row_names,
+    );
+}
+
+
+method split (ColumnLike $factor) {
+    if ($factor->$_DOES('PDL::Factor')) {
+        $factor = $factor->{PDL};
+    }
+    my $uniq_values = $factor->$_call_if_can('uniq')
+      // [ List::AllUtils::uniq( $factor->flatten ) ];
+
+    my @rslt = map {
+        my $indices = PDL::Primitive::which( $factor == $_ );
+        $_ => $self->select_rows($indices);
+    } $uniq_values->flatten;
+
+    return (wantarray ? @rslt : { @rslt });
+}
+
+
 # below lvalue methods are for slice()
 sub _column : lvalue     { my $col = shift->column(@_);     return $col; }
 sub _nth_column : lvalue { my $col = shift->nth_column(@_); return $col; }
 
+method _select_columns (@rest) : lvalue {
+    my $indexer = indexer_s(@rest);
+    return $self if ( not defined $indexer or $indexer->indexer->length == 0 );
+
+    my $indices      = $self->_cindexer_to_indices($indexer);
+    my $column_names = $self->column_names;
+    return ref($self)->new(
+        columns => $indices->map(
+            sub { $column_names->at($_) => $self->_nth_column($_) }
+        ),
+        row_names => $self->row_names
+    );
+}
+
 classmethod _check_slice_args (@rest) {
     state $check_labels =
       Type::Params::compile( Indexer->plus_coercions(IndexerFromLabels) );
@@ -643,142 +698,44 @@ method slice(@rest) : lvalue {
 }
 
 
-method sample ($n) {
-    if ($n > $self->nrow) {
-        die "sample size is larger than nrow";
+method assign ((DataFrame | Piddle) $x) {
+    if ( DataFrame->check($x) ) {
+        unless ( ( $self->shape == $x->shape )->all ) {
+            die "Cannot assign a data frame of different shape.";
+        }
+        for my $name ( $self->names->flatten ) {
+            my $col = $self->at($name);
+            $col .= $x->at($name);
+        }
     }
+    elsif ( $x->$_DOES('PDL') ) {
+        my @dims = $self->dims;
 
-    my @indices = (0 .. $self->nrow-1);
-    my $indices = [ List::AllUtils::sample($n, @indices) ];
-    return $self->select_rows($indices);
-}
-
-
-method which (:$bad_to_val=undef, :$ignore_both_bad=true) {
-    my $coordinates = [ 0 .. $self->ncol - 1 ]->map(
-        fun($cidx)
+        unless ( $x->ndims == 1 and $x->dim(0) == $dims[0] * $dims[1]
+            or $x->ndims == 2
+            and $x->dim(0) == $dims[0]
+            and $x->dim(1) == $dims[1] )
         {
-            my $column = $self->nth_column( $cidx );
-            my $both_bad =
-                $self->DOES('Data::Frame::Role::CompareResult')
-              ? $self->both_bad->nth_column( $cidx )
-              : undef;
-
-            if ( defined $bad_to_val ) {
-                $column = $column->setbadtoval($bad_to_val);
-            }
-
-            my $indices_false = PDL::Primitive::which(
-                defined $both_bad ? ( !$both_bad & $column ) : $column );
-            return $indices_false->unpdl->map( sub { [ $_, $cidx ] } )->flatten;
+            die;
         }
-    );
-    return pdl($coordinates);
+
+        for my $i ( 0 .. $self->length - 1 ) {
+            $self->_nth_column($i) .=
+              $x->flat->slice( pdl( 0 .. $dims[0] - 1 ) + $i * $dims[1] );
+        }
+    }
+    return $self;
 }
 
 
-method merge (DataFrame $df) {
-    my $class   = ref($self);
-    my $columns = [
-        $self->names->map( sub { $_ => $self->column($_) } )->flatten,
-        $df->names->map( sub { $_ => $df->column($_) } )->flatten
-    ];
-    return $class->new(
-        columns   => $columns,
-        row_names => $self->row_names
-    );
-}
-*cbind = \&merge;
-
-method append (DataFrame $df) {
-    if ( $df->nrow == 0 ) {                     # $df is empty
-        return $self->clone();
-    }
-    if ( $self->column_names->length == 0) {    # $self has no columns
-        return $df->clone;
-    }
-
-    my $class   = ref($self);
-    my $columns = $self->names->map(
-        sub {
-            my $col = $self->column($_);
-            # use glue() as PDL's append() cannot handle bad values
-            $_ => $col->glue( 0, $df->column($_) );
-        }
-    );
-    return $class->new( columns => $columns );
-}
-*rbind = \&append;
-
-
-
-method transform ($func) {
-    state $check = Type::Params::compile(
-        (
-            CodeRef | ( HashRef [ Maybe [CodeRef] ] ) |
-              ( CycleTuple [ Str, Maybe [CodeRef] ] )
-        )
-    );
-    ($func) = $check->($func);
-
-    my $class = ref($self);
-
-    my @columns;
-    if ( Ref::Util::is_coderef($func) ) {
-        @columns =
-          $self->names->map( sub {
-            $_ => $func->( $self->column($_), $self );
-          } )->flatten;
-    }
-    else {    # hashref or arrayref
-        my $column_names = $self->names;
-        my $hashref;
-        my @new_column_names;
-        if ( Ref::Util::is_hashref($func) ) {
-            $hashref = $func;
-            @new_column_names =
-              grep { !$self->exists($_) } sort( keys %$hashref );
-        }
-        else {    # arrayref
-            $hashref = {@$func};
-            @new_column_names = grep { !$self->exists($_) } ( pairkeys @$func );
-        }
-
-        @columns = $column_names->map(
-            sub {
-                my $f = exists($hashref->{$_}) ? $hashref->{$_} : sub { $_[0] };
-                $f //= sub { undef };
-                $_ => $f->( $self->column($_), $self );
-            }
-        )->flatten;
-        push @columns,
-          map { my $f = $hashref->{$_}; $_ => $f->( undef, $self ) }
-          @new_column_names;
-    }
-
-    my %columns_to_drop = @columns;
-    %columns_to_drop = pairgrep { not defined $b } %columns_to_drop;
-
-    return $class->new(
-        columns   => [ pairgrep { !exists($columns_to_drop{$a}) } @columns ],
-        row_names => $self->row_names,
-    );
+method is_numeric_column ($column_name_or_idx) {
+    my $column = $self->at($column_name_or_idx);
+    return $self->_is_numeric_column($column);
 }
 
-
-method split (ColumnLike $factor) {
-    if ($factor->$_DOES('PDL::Factor')) {
-        $factor = $factor->{PDL};
-    }
-    my $uniq_values = $factor->$_call_if_can('uniq')
-      // [ List::AllUtils::uniq( $factor->flatten ) ];
-
-    my @rslt = map {
-        my $indices = PDL::Primitive::which( $factor == $_ );
-        $_ => $self->select_rows($indices);
-    } $uniq_values->flatten;
-
-    return (wantarray ? @rslt : { @rslt });
+sub _is_numeric_column {
+    my ($self, $piddle) = @_;
+    return !is_discrete($piddle);
 }
 
 
@@ -827,30 +784,17 @@ method sorti ($by_columns, $ascending=true) {
 }
 
 
-my $sereal = Sereal::Encoder->new();
-
-sub _serialize_row {
-    my ($self, $i, $columns, $columns_isbad) = @_;
-
-    my @row_data =
-      map {
-        my $isbad = $columns_isbad->[$_];
-        ( defined $isbad and $isbad->at($i) )
-          ? ( 1, undef )
-          : ( 0, $columns->[$_]->at($i) );
-      } ( 0 .. $#$columns );
+method _serialize_row ($i) {
+    state $sereal = Sereal::Encoder->new();
+    my @row_data = map { $self->at($_)->at($i) } @{ $self->column_names };
     return $sereal->encode( \@row_data );
 }
 
 method uniq () {
     my %uniq;
     my @uniq_ridx;
-
-    my @columns = map { $self->column($_) } @{ $self->column_names };
-    my @columns_isbad = map { $_->badflag ? $_->isbad : undef } @columns;
-
     for my $i ( 0 .. $self->nrow - 1 ) {
-        my $key = $self->_serialize_row($i, \@columns, \@columns_isbad);
+        my $key = $self->_serialize_row($i);
         unless ( exists $uniq{$key} ) {
             $uniq{$key} = 1;
             push @uniq_ridx, $i;
@@ -861,13 +805,10 @@ method uniq () {
 
 
 method id () {
-    my @columns = map { $self->column($_) } @{ $self->column_names };
-    my @columns_isbad = map { $_->badflag ? $_->isbad : undef } @columns;
-
     my %uniq_serialized;
     my @uniq_rindices;
     for my $ridx ( 0 .. $self->nrow - 1 ) {
-        my $key = $self->_serialize_row($ridx, \@columns, \@columns_isbad);
+        my $key = $self->_serialize_row($ridx);
         if ( not exists $uniq_serialized{$key} ) {
             $uniq_serialized{$key} = [];
             push @uniq_rindices, $ridx;
@@ -890,67 +831,37 @@ method id () {
 }
 
 
-method assign ((DataFrame | Piddle) $x) {
-    if ( DataFrame->check($x) ) {
-        unless ( ( $self->shape == $x->shape )->all ) {
-            die "Cannot assign a data frame of different shape.";
-        }
-        for my $name ( $self->names->flatten ) {
-            my $col = $self->column($name);
-            $col .= $x->column($name);
-        }
-    }
-    elsif ( $x->$_DOES('PDL') ) {
-        my @dims = $self->dims;
+method copy () { 
+    return ref($self)->new(
+        columns   => $self->names->map( sub { $_ => $self->at($_)->copy } ), 
+        row_names => $self->row_names->copy
+    );
+}
+*clone = \&copy;
 
-        unless ( $x->ndims == 1 and $x->dim(0) == $dims[0] * $dims[1]
-            or $x->ndims == 2
-            and $x->dim(0) == $dims[0]
-            and $x->dim(1) == $dims[1] )
+
+method which (:$bad_to_val=undef, :$ignore_both_bad=true) {
+    my $coordinates = [ 0 .. $self->ncol - 1 ]->map(
+        fun($cidx)
         {
-            die;
+            my $column = $self->at( indexer_i( [$cidx] ) );
+            my $both_bad =
+                $self->DOES('Data::Frame::Role::CompareResult')
+              ? $self->both_bad->at( indexer_i( [$cidx] ) )
+              : undef;
+
+            if ( defined $bad_to_val ) {
+                $column = $column->setbadtoval($bad_to_val);
+            }
+
+            my $indices_false = PDL::Primitive::which(
+                defined $both_bad ? ( !$both_bad & $column ) : $column );
+            return $indices_false->unpdl->map( sub { [ $_, $cidx ] } )->flatten;
         }
-
-        for my $i ( 0 .. $self->length - 1 ) {
-            $self->_nth_column($i) .=
-              $x->flat->slice( pdl( 0 .. $dims[0] - 1 ) + $i * $dims[1] );
-        }
-    }
-    return $self;
+    );
+    return pdl($coordinates);
 }
 
-
-method is_numeric_column ($column_name_or_idx) {
-    my $column = $self->at($column_name_or_idx);
-    return $self->_is_numeric_column($column);
-}
-
-sub _is_numeric_column {
-    my ($self, $piddle) = @_;
-    return !is_discrete($piddle);
-}
-
-
-method drop_bad (:$how='any') {
-    state $check = Type::Params::compile( Enum [qw(all any)] );
-    ($how) = $check->($how);
-
-    my $f = sub {
-        my ($op) = @_;
-        return reduce {
-            my $col = $self->column($b);
-            $col->badflag ? $op->( $a, $col->isbad ) : $a;
-        } PDL::Core::zeros( $self->nrow ), $self->names->flatten;
-    };
-
-    my $isbad =
-        $how eq 'any'
-      ? $f->( sub { $_[0] | $_[1] } )
-      : $f->( sub { $_[0] & $_[1] } );
-    return $self->select_rows( PDL::Primitive::which( !$isbad ) );
-}
-
-
 method _compare ($other, $mode) {
     my $class = ref($self);
 
@@ -986,7 +897,7 @@ method _compare ($other, $mode) {
         my ( $col, $x ) = @_;
         my $a = $col->abs;
         my $b = ref($x) ? $x->abs : abs($x);
-        return (ifelse( $a > $b, $a, $b ) * abs($TOLERANCE_REL));
+        return ifelse( $a > $b, $a, $b ) * $TOLERANCE_REL;
     };
 
     state $fcompare_float = {
@@ -1012,7 +923,7 @@ method _compare ($other, $mode) {
     my $compare_column = sub {
         my ( $name, $x ) = @_;
 
-        my $col = $self->column($name);
+        my $col = $self->at($name);
 
         my $fcompare;
         if ( $self->is_numeric_column($name) ) {
@@ -1049,7 +960,7 @@ method _compare ($other, $mode) {
         }
         $result_columns = {
             $self->names->map(
-                sub { $_ => [ $compare_column->( $_, $other->column($_) ) ]; }
+                sub { $_ => [ $compare_column->( $_, $other->at($_) ) ]; }
             )->flatten
         };
     }
@@ -1081,7 +992,6 @@ sub _column_helper {
 	Data::Frame::Column::Helper->new( dataframe => $self );
 }
 
-
 1;
 
 __END__
@@ -1096,11 +1006,11 @@ Data::Frame - data frame implementation
 
 =head1 VERSION
 
-version 0.0059_01
+version 0.0041
 
 =head1 STATUS
 
-This library is currently experimental.
+This library is current experimental.
 
 =head1 SYNOPSIS
 
@@ -1114,6 +1024,7 @@ This library is currently experimental.
                 y => ( sequence(4) >= 2 ) ,
                 x => [ qw/foo bar baz quux/ ],
             ] );
+
     say $df;
     # ---------------
     #     z  y  x
@@ -1124,22 +1035,8 @@ This library is currently experimental.
     #  3  4  1  quux
     # ---------------
 
-    say $df->at(0);         # [1 2 3 4]
-    say $df->at(0)->length; # 4
-
-    say $df->at('x');       # [foo bar baz quux]
-    say $df->{x};           # same as above
-
-    say $df->select_columns([qw(x y)]);
-    # -----------
-    #     x    y
-    # -----------
-    #  0  foo  0
-    #  1  bar  0
-    #  2  baz  1
-    #  3  quux 1
-    # -----------
-    say $df->{[qw(x y)]};   # same as above
+    say $df->at(0);
+    # [1 2 3 4]
 
     say $df->select_rows( 3,1 );
     # ---------------
@@ -1149,7 +1046,6 @@ This library is currently experimental.
     #  1  2  0  bar
     # ---------------
 
-    # update data
     $df->slice( [0,1], ['z', 'y'] ) .= pdl( 4,3,2,1 );
     say $df;
     # ---------------
@@ -1169,13 +1065,6 @@ So here I release my L<Alt> implenmentation.
 This implements a data frame container that uses L<PDL> for individual columns.
 As such, it supports marking missing values (C<BAD> values).
 
-=head2 Document Conventions
-
-Function signatures in docs of this library follow the
-L<Function::Parameters> conventions, for example,
-
-    myfunc(Type1 $positional_parameter, Type2 :$named_parameter)
-
 =head1 CONSTRUCTION
 
     new( (ArrayRef | HashRef) :$columns,
@@ -1214,7 +1103,7 @@ C<HashRef> (sorted with a stringwise C<cmp>).
 
 =back
 
-=head1 METHODS / BASIC
+=head1 METHODS
 
 =head2 string
 
@@ -1222,33 +1111,41 @@ C<HashRef> (sorted with a stringwise C<cmp>).
 
 Returns a string representation of the C<Data::Frame>.
 
-=head2 ncol / length / number_of_columns 
+=head2 number_of_columns
 
-These methods are same,
-
-    # returns Int
-    ncol()
-    length()
     number_of_columns() # returns Int
 
 Returns the count of the number of columns in the C<Data::Frame>.
 
-=head2 nrow / number_of_rows
+=head2 ncol
 
-These methods are same,
+    ncol()
+
+This is same as C<number_of_columns>.
+
+=head2 length
+
+    length()
+
+This is same as C<number_of_columns>.
+
+=head2 number_of_rows
 
-    # returns Int
-    nrow()
     number_of_rows() # returns Int
 
 Returns the count of the number of rows in the C<Data::Frame>.
 
+=head2 nrow
+
+    nrow()
+
+This is same as C<number_of_rows>.
+
 =head2 dims
 
     dims()
 
-Returns the dimensions of the data frame object, in an array of
-C<($nrow, $ncol)>.
+Returns the dimensions of the data frame object, in an array of C<($nrow, $ncol)>.
 
 =head2 shape
 
@@ -1289,24 +1186,13 @@ In-place delete column given by C<$col_name>.
 
 In-place rename columns.
 
-It can take either, 
+=head2 select_columns
 
-=over 4
+    select_columns($indexer) 
 
-=item *
+Returns a new data frame object which has the columns selected by C<$indexer>.
 
-A hashref of key mappings.
-
-If a keys does not exist in the mappings, it would not be renamed. 
-
-=item *
-
-A coderef which transforms each key.
-
-=back
-
-    $df->rename( { $from_key => $to_key, ... } );
-    $df->rename( sub { $_[0] . 'foo' } );
+If a given argument is non-indexer, it would coerce it by C<indexer_s()>.
 
 =head2 set
 
@@ -1320,22 +1206,18 @@ Sets data to column. If C<$col_name> does not exist, it would add a new column.
 
 Returns true if the data frame has no rows.
 
-=head2 names / col_names / column_names
+=head2 nth_columm
 
-These methods are same
+    number_of_rows(Int $n) # returns a column
 
-    # returns ArrayRef
-    names()
-    names( $new_column_names )
-    names( @new_column_names )
+Returns column number C<$n>. Supports negative indices (e.g., $n = -1 returns
+the last column).
 
-    col_names()
-    col_names( $new_column_names )
-    col_names( @new_column_names )
+=head2 column_names
 
-    column_names()
-    column_names( $new_column_names )
-    column_names( @new_column_names )
+    column_names() # returns an ArrayRef
+
+    column_names( @new_column_names ) # returns an ArrayRef
 
 Returns an C<ArrayRef> of the names of the columns.
 
@@ -1343,15 +1225,29 @@ If passed a list of arguments C<@new_column_names>, then the columns will be
 renamed to the elements of C<@new_column_names>. The length of the argument
 must match the number of columns in the C<Data::Frame>.
 
+=head2 col_names
+
+    col_names($new_names)
+
+This is same as C<column_names>.
+
+=head2 names
+
+    names($new_names)
+
+This is same as C<column_names>.
+
 =head2 row_names
 
-    # returns a PDL::SV
-    row_names()
-    row_names( Array @new_row_names )
-    row_names( ArrayRef $new_row_names )
-    row_names( PDL $new_row_names )
+    row_names() # returns a PDL
 
-Returns an C<PDL::SV> of the names of the rows.
+    row_names( Array @new_row_names ) # returns a PDL
+
+    row_names( ArrayRef $new_row_names ) # returns a PDL
+
+    row_names( PDL $new_row_names ) # returns a PDL
+
+Returns an C<ArrayRef> of the names of the columns.
 
 If passed a argument, then the rows will be renamed. The length of the argument
 must match the number of rows in the C<Data::Frame>.
@@ -1362,13 +1258,6 @@ must match the number of rows in the C<Data::Frame>.
 
 Returns the column with the name C<$column_name>.
 
-=head2 nth_column
-
-    number_of_rows(Int $n) # returns a column
-
-Returns column number C<$n>. Supports negative indices (e.g., $n = -1 returns
-the last column).
-
 =head2 add_columns
 
     add_columns( Array @column_pairlist )
@@ -1382,50 +1271,6 @@ Adds all the columns in C<@column_pairlist> to the C<Data::Frame>.
 Adds a single column to the C<Data::Frame> with the name C<$name> and data
 C<$data>.
 
-=head2 copy / clone
-
-These methods are same,
-
-    copy()
-    clone()
-
-Make a deep copy of this data frame object.
-
-=head2 summary
-
-    summary($percentiles=[0.25, 0.75])
-
-Generate descriptive statistics that summarize the central tendency,
-dispersion and shape of a dataset’s distribution, excluding C<BAD> values.
-
-Analyzes numeric datetime columns only. For other column types like
-C<PDL::SV> and C<PDL::Factor> gets only good value count.
-Returns a data frame of the summarized statistics.
-
-Parameters:
-
-=over 4
-
-=item *
-
-$percentiles
-
-The percentiles to include in the output. All should fall between 0 and 1.
-The default is C<[.25, .75]>, which returns the 25th, 50th, and 75th
-percentiles (median is always automatically included).
-
-=back
-
-=head1 METHODS / SELECTING AND INDEXING
-
-=head2 select_columns
-
-    select_columns($indexer) 
-
-Returns a new data frame object which has the columns selected by C<$indexer>.
-
-If a given argument is non-indexer, it would coerce it by C<indexer_s()>.
-
 =head2 select_rows
 
     select_rows( Indexer $indexer)
@@ -1444,49 +1289,6 @@ values in the child data frame columns will appear in the parent data frame.
 
 If no indices are given, a C<Data::Frame> with no rows is returned.
 
-=head2 head
-
-    head( Int $n=6 )
-
-If $n ≥ 0, returns a new C<Data::Frame> with the first $n rows of the
-C<Data::Frame>.
-
-If $n < 0, returns a new C<Data::Frame> with all but the last -$n rows of the
-C<Data::Frame>.
-
-See also: R's L<head|https://stat.ethz.ch/R-manual/R-devel/library/utils/html/head.html> function.
-
-=head2 tail
-
-    tail( Int $n=6 )
-
-If $n ≥ 0, returns a new C<Data::Frame> with the last $n rows of the
-C<Data::Frame>.
-
-If $n < 0, returns a new C<Data::Frame> with all but the first -$n rows of the
-C<Data::Frame>.
-
-See also: R's L<tail|https://stat.ethz.ch/R-manual/R-devel/library/utils/html/head.html> function.
-
-=head2 slice
-
-    my $subset1 = $df->slice($row_indexer, $column_indexer);
-
-    # Note that below two cases are different.
-    my $subset2 = $df->slice($column_indexer);
-    my $subset3 = $df->slice($row_indexer, undef);
-
-Returns a new dataframe object which is a slice of the raw data frame.
-
-This method returns an lvalue which allows PDL-like C<.=> assignment for
-changing a subset of the raw data frame. For example,
-
-    $df->slice($row_indexer, $column_indexer) .= $another_df;
-    $df->slice($row_indexer, $column_indexer) .= $piddle;
-
-If a given argument is non-indexer, it would try guessing if the argument
-is numeric or not, and coerce it by either C<indexer_s()> or C<indexer_i()>.
-
 =head2 sample
 
     sample($n)
@@ -1495,30 +1297,25 @@ Get a random sample of rows from the data frame object, as a new data frame.
 
     my $sample_df = $df->sample(100);
 
-=head2 which
-
-    which(:$bad_to_val=undef, :$ignore_both_bad=true)
-
-Returns a pdl of C<[[col_idx, row_idx], ...]>, like the output of
-L<PDL::Primitive/whichND>.
-
-=head1 METHODS / MERGE
-
-=head2 merge / cbind
-
-These methods are same,
+=head2 merge
 
     merge($df)
+
+=head2 cbind
+
     cbind($df)
 
-=head2 append / rbind
+This is same as C<merge()>.
 
-These methods are same,
+=head2 append
 
     append($df)
+
+=head2 rbind
+
     rbind($df)
 
-=head1 METHODS / TRANSFORMATION AND GROUPING
+This is same as C<append()>.
 
 =head2 transform
 
@@ -1533,25 +1330,21 @@ C<$func> can be one of the following,
 
 =item *
 
-A function coderef.
-
-It would be applied to all columns.
+A function coderef. It would be applied to all columns.
 
 =item *
 
-A hashref of C<{ $column_name =E<gt> $coderef, ... }>
+A hashref of C<{ $column_name =E<gt> $coderef, ... }>. It allows to apply
 
-It allows to apply the function to the specified columns. The raw data
-frame's columns not existing in the hashref be retained unchanged. Hashref
-keys not yet existing in the raw data frame can be used for creating new
-columns.
+the function to the specified columns. The raw data frame's columns not 
+existing in the hashref be retained unchanged. Hashref keys not yet
+existing in the raw data frame can be used for creating new columns.
 
 =item *
 
-An arrayref like C<[ $column_name =E<gt> $coderef, ... ]>
+An arrayref like C<[ $column_name =E<gt> $coderef, ... ]>. In this mode
 
-In this mode it's similar as the hasref above, but newly added columns
-would be in order.
+it's similar as the hasref above, but newly added columns would be in order.
 
 =back
 
@@ -1608,6 +1401,49 @@ values in C<$factor>.
 
 Note that C<$factor> does not necessarily to be PDL::Factor.
 
+=head2 slice
+
+    my $subset1 = $df->slice($row_indexer, $column_indexer);
+
+    # Note that below two cases are different.
+    my $subset2 = $df->slice($column_indexer);
+    my $subset3 = $df->slice($row_indexer, undef);
+
+Returns a new dataframe object which is a slice of the raw data frame.
+
+This method returns an lvalue which allows PDL-like C<.=> assignment for
+changing a subset of the raw data frame. For example,
+
+    $df->slice($row_indexer, $column_indexer) .= $another_df;
+    $df->slice($row_indexer, $column_indexer) .= $piddle;
+
+If a given argument is non-indexer, it would try guessing if the argument
+is numeric or not, and coerce it by either C<indexer_s()> or C<indexer_i()>.
+
+=head2 assign
+
+    assign( (DataFrame|Piddle) $x )
+
+Assign another data frame or a piddle to this data frame for in-place change.
+
+C<$x> can be,
+
+=over 4
+
+*A data frame object having the same dimensions and column names as C<$self>.
+*A piddle having the same number of elements as C<$self>.
+
+=back
+
+This method is internally used by the C<.=> operation, below are same,
+
+    $df->assign($x);
+    $df .= $x;
+
+=head2 is_numeric_column
+
+    is_numeric_column($column_name_or_idx)
+
 =head2 sort
 
     sort($by_columns, $ascending=true)
@@ -1636,54 +1472,36 @@ are from the first occurrance of each unique row in the raw data frame.
 
 Compute a unique numeric id for each unique row in a data frame.
 
-=head1 METHODS / OTHERS
+=head2 copy 
 
-=head2 assign
+    copy()
 
-    assign( (DataFrame|Piddle) $x )
+Make a deep copy of this data frame object.
 
-Assign another data frame or a piddle to this data frame for in-place change.
+=head2 clone
 
-C<$x> can be,
+    clone()
 
-=over 4
+This is same as C<copy()>.
 
-=item *
+=head2 which
 
-A data frame object having the same dimensions and column names as C<$self>.
+    which(:$bad_to_val=undef, :$ignore_both_bad=true)
 
-=item *
-
-A piddle having the same number of elements as C<$self>.
-
-=back
-
-This method is internally used by the C<.=> operation, below are same,
-
-    $df->assign($x);
-    $df .= $x;
-
-=head2 is_numeric_column
-
-    is_numeric_column($column_name_or_idx)
-
-=head2 drop_bad
-
-    drop_bad(:$how='any')
-
-Returns a new data frame with rows with BAD values dropped.
+Returns a pdl of C<[[col_idx, row_idx], ...]>, like the output of
+L<PDL::Primitive/whichND>.
 
 =head1 MISCELLANEOUS FEATURES
 
-=head2 Serialization
+=head2 SERIALIZATION
 
 See L<Data::Frame::IO::CSV>
 
-=head2 Syntax Sugar
+=head2 SYNTAX SUGAR
 
 See L<Data::Frame::Partial::Sugar>
 
-=head2 Tidy Evaluation
+=head2 TIDY EVALUATION
 
 This feature is somewhat similar to R's tidy evaluation.
 
@@ -1691,11 +1509,11 @@ See L<Data::Frame::Partial::Eval>.
 
 =head1 VARIABLES
 
-=head2 doubleformat
+=head1 doubleformat
 
 This is used when stringifying the data frame. Default is C<'%.8g'>.
 
-=head2 TOLERANCE_REL
+=head1 TOLERANCE_REL
 
 This is the relative tolerance used when comparing numerical values of two
 data frames.
@@ -1707,10 +1525,10 @@ Default is C<undef>, which means no tolerance at all. You can set it like,
 
 =over 4
 
-=item * L<Data::Frame::Examples>
-
 =item * L<Alt>
 
+=item * L<Data::Frame::Examples>
+
 =item * L<PDL>
 
 =item * L<R manual: data.frame|https://stat.ethz.ch/R-manual/R-devel/library/base/html/data.frame.html>.
@@ -1735,7 +1553,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014, 2019-2021 by Zakariyya Mughal, Stephan Loyd.
+This software is copyright (c) 2014, 2019 by Zakariyya Mughal, Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
