@@ -12,7 +12,7 @@ use PDL::SV        ();
 use PDL::DateTime  ();
 use PDL::Types     ();
 
-use List::AllUtils qw(uniq);
+use Data::Munge qw(elem);
 use Package::Stash;
 use Ref::Util qw(is_plain_arrayref is_plain_hashref);
 use Scalar::Util qw(openhandle looks_like_number);
@@ -87,11 +87,11 @@ classmethod from_csv ($file, :$header=true, :$sep=",", :$quote='"',
 
     my @row_names;
     my $rows = $csv->getline_all($fh);
-
-    my $offset = $row_names_from_first_column ? 1 : 0;
     for my $row (@$rows) {
+        my $offset = 0;
         if ($row_names_from_first_column) {
             push @row_names, $row->[0];
+            $offset = 1;
         }
         for my $i ( 0 .. $#col_names ) {
             my $col = $col_names[$i];
@@ -112,30 +112,20 @@ classmethod from_csv ($file, :$header=true, :$sep=",", :$quote='"',
     }
 
     state $additional_type_to_piddle = {
-        datetime => \&Data::Frame::Util::_datetime_from_arrayref,
-        factor   => \&Data::Frame::Util::_factor_from_arrayref,
-        logical  => \&Data::Frame::Util::_logical_from_arrayref,
-        pdlsv    => \&Data::Frame::Util::_pdlsv_from_arrayref,
+        datetime => sub { PDL::DateTime->new_from_datetime($_[0]) },
+        factor   => sub { PDL::Factor->new($_[0]) },
+        logical  => sub { PDL::Logical->new($_[0]) },
     };
-
     my $package_pdl_core = Package::Stash->new('PDL::Core');
-    my $to_piddle        = sub {
+    my $to_piddle = sub {
         my ($name) = @_;
         my $x = $columns{$name};
 
         if ( my $type = $dtype->{$name} ) {
-            my $piddle_maker = $additional_type_to_piddle->{$type};
-            unless ($piddle_maker) {
-                if ( my $f = $package_pdl_core->get_symbol("&$type") ) {
-                    $piddle_maker = sub {
-                        my ( $x, $na ) = @_;
-                        Data::Frame::Util::_numeric_from_arrayref( $x, $na,
-                            $f );
-                    };
-                }
-            }
-            if ($piddle_maker) {
-                return $piddle_maker->( $x, $na );
+            my $f_new = $additional_type_to_piddle->{$type}
+              // $package_pdl_core->get_symbol("&$type");
+            if ($f_new) {
+                return $f_new->($x);
             }
             else {
                 die "Invalid data type '$type'";
@@ -187,19 +177,14 @@ method to_csv ($file, :$sep=',', :$quote='"', :$na='NA',
     }
 
     # a hash to store isbad info for each column
-    my %isbad = map { $_ => $self->column($_)->isbad; } ( $self->names->flatten );
+    my %is_bad = map { $_ => $self->at($_)->isbad; } ( $self->names->flatten );
 
     for ( my $i = 0 ; $i < $self->nrow ; $i++ ) {
         my @row = (
             ( $row_names ? $row_names_data->at($i) : () ),
             (
-                map {
-                    $self->_format_cell(
-                        $self->column($_), $i,
-                        na    => $na,
-                        isbad => $isbad{$_}
-                    );
-                } @{ $self->names }
+                map { $is_bad{$_}->at($i) ? $na : $self->at($_)->at($i); }
+                  @{ $self->names }
             )
         );
         $csv->print( $fh, \@row );
@@ -220,7 +205,7 @@ Data::Frame::IO::CSV - Partial class for data frame's conversion from/to CSV
 
 =head1 VERSION
 
-version 0.0059_01
+version 0.0041
 
 =head1 METHODS
 
@@ -241,25 +226,18 @@ Some of the parameters are explained below,
 
 =item *
 
-C<$file>
+C<$file> can be a file name string, a Path::Tiny object, or an opened file
 
-This can be a file name string, a Path::Tiny object, or an opened file handle.
+handle.
 
 =item *
 
-C<$dtype>
+C<$dtype> is a hashref associating column names to their types. Types
 
-A hashref associating column names to their types.
-Types can be the PDL type names like C<"long">, C<"double">, or C<"pdlsv">, 
-C<"factor">, C<"datetime">, C<"logical">.
-If a column is not specified in C<$dtype>, its type is automatically decided.
-
-=item *
-
-C<$na>
-
-An arrayref of strings which are to be interpreted as C<BAD> values.
-Blank fields are also considered to be missing value in logical and numeric fields.
+can be the PDL type names like C<"long">, C<"double">, or names of some PDL's
+derived class like C<"PDL::SV">, C<"PDL::Factor">, C<"PDL::DateTime">. If a
+column is not specified in C<$dtype>, its type would be automatically
+decided.
 
 =back
 
@@ -286,7 +264,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014, 2019-2021 by Zakariyya Mughal, Stephan Loyd.
+This software is copyright (c) 2014, 2019 by Zakariyya Mughal, Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
