@@ -2,7 +2,8 @@ package Data::Frame::PDL;
 
 # ABSTRACT: A mixin to add some methods to PDL
 
-use 5.016;
+use 5.010;
+use strict;
 use warnings;
 
 use Role::Tiny;
@@ -12,6 +13,10 @@ use PDL::Core qw(pdl);
 use PDL::Primitive qw(which);
 use POSIX qw(ceil);
 use Safe::Isa;
+use Types::Standard qw(Int);
+use Type::Params;
+
+my $PositiveInt = Int->where( sub { $_ > 0 } );
 
 
 sub length { $_[0]->dim(0); }
@@ -61,7 +66,7 @@ sub repeat {
         $p = $class->new( [ ( @{ $self->unpdl } ) x $n ] );
     }
     elsif ( $self->$_DOES('PDL::Factor') ) {
-        $p = $class->new( $self, levels => $self->levels );
+        $p = $class->new( $self->levels, levels => $self->levels );
         $p->{PDL} = $repeat->( $p->{PDL}, $n );
     }
     else {
@@ -116,9 +121,9 @@ sub as_pdlsv {
 sub id {
     my ($self) = @_;
 
-    my %uniq_values;    # value to row indices
-    my @uniq_indices;   # first index for each set of uniq values
-    for my $ridx ( which($self->isgood)->list ) {
+    my %uniq_values;
+    my @uniq_indices;
+    for my $ridx ( 0 .. $self->length - 1 ) {
         my $value = $self->at($ridx);
         if ( not exists $uniq_values{$value} ) {
             $uniq_values{$value} = [];
@@ -130,8 +135,7 @@ sub id {
     my %index_to_value = pairmap { $b->[0] => $a } %uniq_values;
 
     my $rslt = PDL::Core::zeros( $self->length );
-    $rslt .= -1;    # for BAD values
-    for my $i ( 0 .. $#uniq_indices ) {
+    for my $i ( 1 .. $#uniq_indices ) {
         my $value =
           $index_to_value{ $uniq_indices[ $i ] }; 
         my $indices = $uniq_values{$value};
@@ -154,51 +158,37 @@ Data::Frame::PDL - A mixin to add some methods to PDL
 
 =head1 VERSION
 
-version 0.0059_01
+version 0.0041
 
 =head1 DESCRIPTION
 
 This module provides a role that can add a few methods to the PDL class.
 
-It's an internal module used by the Data::Frame library to add a few methods
-to the PDL class. Do not directly use this module in your code, as the
-module name may change in future.
+It's an internal module used by Data::Frame. Do not use it in your code!
 
 =head1 METHODS
 
-=head2 length
-
-    length()
+=head2 length()
 
 Returns the length of the first dimension.
 
-=head2 diff
+=head2 diff($lag=1)
 
-    diff($lag=1)
-
-=head2 flatten
-
-    flatten()
+=head2 flatten()
 
 This is same as C<@{$self-E<gt>unpdl}>.
 
-=head2 flatten_deep
-
-    flatten_deep()
+=head2 flatten_deep()
 
 This is same as C<list()>.
 
-=head2 repeat
-
-    repeat($n)
+=head2 repeat($n)
 
 Repeat on the first dimension for C<$n> times.
 
 Only works with 1D piddle.  
 
-=head2 repeat_to_length
-
-    repeat_to_length($length)
+=head2 repeat_to_length($length)
 
 Repeat to have the given length.
 
@@ -206,8 +196,6 @@ Only works with 1D piddle.
 
 =head2 id
 
-    id()
-
 Compute a unique numeric id for each element in a piddle.
 
 =head1 SEE ALSO
@@ -230,7 +218,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014, 2019-2021 by Zakariyya Mughal, Stephan Loyd.
+This software is copyright (c) 2014, 2019 by Zakariyya Mughal, Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
