@@ -6,16 +6,13 @@ use Data::Frame::Role;
 use namespace::autoclean;
 
 package Tie::Data::Frame {
-    use Scalar::Util qw(weaken);
     use Types::PDL qw(Piddle);
     use Types::Standard qw(ArrayRef Value);
     use Type::Params;
 
     sub new {
         my ($class, $object) = @_;
-        my $self = bless( { _object => $object }, $class);
-        weaken( $self->{_object} );
-        return $self;
+        return bless( { _object => $object }, $class);
     }
 
     sub TIEHASH {
@@ -107,7 +104,7 @@ Data::Frame::Partial::Sugar - Partial class for data frame syntax sugar
 
 =head1 VERSION
 
-version 0.0059_01
+version 0.0041
 
 =head1 SYNOPSIS
 
@@ -123,9 +120,6 @@ version 0.0059_01
 
 =head1 DESCRIPTION
 
-Do not use this module in your code. This is only internally used by
-L<Data::Frame>.
-
 =head1 SEE ALSO
 
 L<Data::Frame>
@@ -146,7 +140,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014, 2019-2021 by Zakariyya Mughal, Stephan Loyd.
+This software is copyright (c) 2014, 2019 by Zakariyya Mughal, Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
