@@ -3,11 +3,25 @@ package Data::Frame::Rlike;
 use strict;
 use warnings;
 
-use Data::Frame::Util qw(dataframe factor logical);
-
-use parent qw(Exporter::Tiny);
+use Exporter 'import';
 our @EXPORT = qw(dataframe factor logical);
 
+use Data::Frame;
+use PDL::Factor ();
+use PDL::Logical ();
+
+sub dataframe {
+	Data::Frame->new( columns => \@_ );
+}
+
+sub factor {
+	PDL::Factor->new(@_);
+}
+
+sub logical {
+	PDL::Logical->new(@_);
+}
+
 1;
 
 __END__
@@ -22,7 +36,7 @@ Data::Frame::Rlike
 
 =head1 VERSION
 
-version 0.0059_01
+version 0.0041
 
 =head1 DESCRIPTION
 
@@ -48,7 +62,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014, 2019-2021 by Zakariyya Mughal, Stephan Loyd.
+This software is copyright (c) 2014, 2019 by Zakariyya Mughal, Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
