@@ -5,7 +5,6 @@ use Data::Frame::Role;
 
 use List::AllUtils;
 
-
 method head($n=6) {
 	my ($start, $stop);
 	if( $n < 0 ) {
@@ -58,7 +57,7 @@ Data::Frame::Role::Rlike
 
 =head1 VERSION
 
-version 0.0059_01
+version 0.0041
 
 =head1 SYNOPSIS
 
@@ -68,7 +67,7 @@ Data::Frame::Role::Rlike - role to provide R-like methods for Data::Frame
 
 =head2 head
 
-    head( Int $n=6 )
+    head( Int $n )
 
 If $n ≥ 0, returns a new C<Data::Frame> with the first $n rows of the
 C<Data::Frame>.
@@ -80,7 +79,7 @@ See also: R's L<head|https://stat.ethz.ch/R-manual/R-devel/library/utils/html/he
 
 =head2 tail
 
-    tail( Int $n=6 )
+    tail( Int $n )
 
 If $n ≥ 0, returns a new C<Data::Frame> with the last $n rows of the
 C<Data::Frame>.
@@ -135,7 +134,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014, 2019-2021 by Zakariyya Mughal, Stephan Loyd.
+This software is copyright (c) 2014, 2019 by Zakariyya Mughal, Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
