@@ -13,27 +13,26 @@ use Import::Into;
 use Carp;
 use Data::Dumper ();
 use Function::Parameters 2.001003;
-use PDL::Lite ();   # PDL::Lite is the minimal to get PDL work
-use Ref::Util 0.204           ();
-use Safe::Isa 1.000009        ();
-use Syntax::Keyword::Try 0.09 ();
-use boolean ();
+use PDL::Lite;    # PDL::Lite is the minimum required to have a working PDL
+use Ref::Util    ();
+use Safe::Isa 1.000009 ();
+use Syntax::Keyword::Try ();
+use boolean              ();
 
 use Moo 2.003004 ();
 use Moo::Role ();
 
 use List::AllUtils qw(uniq);
 
-use Data::Frame::Autobox ();
+use Moose::Autobox 0.16;
+for my $type (qw(Hash Array)) {
+    Moose::Autobox->mixin_additional_role(
+        uc($type) => "Data::Frame::Autobox::$type" );
+}
 
 use Role::Tiny ();
 Role::Tiny->apply_roles_to_package( 'PDL', 'Data::Frame::PDL' );
 
-# PDL versions between 2.040 and 2.054 has not PDL::Core::at_c
-unless (defined &PDL::Core::at_c) {
-    *PDL::Core::at_c = \&{"PDL::Core::at_bad_c"};
-}
-
 sub import {
     my ( $class, @tags ) = @_;
 
@@ -68,7 +67,7 @@ sub _import_tag {
         Syntax::Keyword::Try->import::into($target);
         boolean->import::into($target);
 
-        Data::Frame::Autobox->import::into($target);
+        Moose::Autobox->import::into($target);
     }
     elsif ( $tag eq ':class' ) {
         $class->_import_tag( $target, ':base' );
@@ -105,7 +104,7 @@ Data::Frame::Setup - Import stuffs into Data::Frame classes
 
 =head1 VERSION
 
-version 0.0059_01
+version 0.0041
 
 =head1 SYNOPSIS
 
@@ -137,7 +136,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014, 2019-2021 by Zakariyya Mughal, Stephan Loyd.
+This software is copyright (c) 2014, 2019 by Zakariyya Mughal, Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
