@@ -7,13 +7,13 @@ use warnings;
 
 use failures qw/levels::mismatch levels::number/;
 
-use PDL::Lite ();   # PDL::Lite is the minimal to get PDL work
 use PDL::Core qw(pdl);
 use PDL::Primitive qw(which);
-
+use Data::Rmap qw(rmap);
 use Module::Load;
 use Ref::Util qw(is_plain_arrayref);
 use Safe::Isa;
+use Storable qw(dclone);
 use Scalar::Util qw(blessed);
 use Type::Params;
 use Types::Standard qw(slurpy ArrayRef ConsumerOf Int);
@@ -25,11 +25,42 @@ use Class::Method::Modifiers;
 use Role::Tiny::With;
 with qw(PDL::Role::Enumerable);
 
+use Devel::OverloadInfo qw(overload_info);
+
+my $overload_info;
+my $super_dotassign;
+
+BEGIN {
+    my $overload_info = overload_info('PDL');
+    $super_dotassign = $overload_info->{'.='}{code};
+}
+
 use overload
-  '==' => \&_eq,
-  'eq' => \&_eq,
-  '!=' => \&_ne,
-  'ne' => \&_ne,
+  '==' => \&equal,
+  '!=' => \&not_equal,
+#  '<'  => \&_lt,
+#  'lt' => \&_lt,
+#  '<=' => \&_le,
+#  'le' => \&_le,
+#  '>'  => \&_gt,
+#  'gt' => \&_gt,
+#  '>=' => \&_ge,
+#  'ge' => \&_ge,
+#
+#  '.=' => sub {
+#    my ( $self, $other, $swap ) = @_;
+#
+#    unless ( $other->$_DOES('PDL::SV') ) {
+#        return $super_dotassign->( $self, $other, $swap );
+#    }
+#
+#    my $internal = $self->_internal;
+#    for my $i ( 0 .. $other->dim(0) - 1 ) {
+#        my $idx = PDL::Core::at( $self, $i );
+#        $internal->[$idx] = $other->at($i);
+#    }
+#    return $self;
+#  },
   fallback => 1;
 
 # after stringifiable role is added, the string method will exist
@@ -92,28 +123,21 @@ sub new {
             return $data->copy;
         }
 
-        #TODO: this does not support ND piddle yet.
-
         # reorder levels
         my @levels      = @{ delete $opt{levels} };
         my @integer_old = $data->{PDL}->list;
-        my @levels_old = @{ $data->levels };
-        my %levels_new = do {
-            my $i = 0;
-            map { $_ => $i++ } @levels;
-        };
-        my @integer_new = do {
-            no warnings 'numeric';
-            map {
-                my $level = $levels_old[$_];
-                defined $level ? $levels_new{$level} : -1;
-            } @integer_old;
-        };
+        my $i           = 0;
+        my %levels_old  = map { $i++ => $_ } @{ $data->levels };
+        $i = 0;
+        my %levels_new  = map { $_ => $i++ } @levels;
+        my @integer_new = map {
+            my $enum = $levels_old{$_};
+            defined $enum ? $levels_new{$enum} : -1;
+        } @integer_old;
 
-        my $new = $class->new( [ $levels[0] ], levels => \@levels, %opt );
+        my $new = $class->new( \@levels, levels => \@levels, %opt );
         my $p = PDL::Core::indx( \@integer_new );
-        $p = $p->setbadif( $data->isbad ) if $data->badflag;
-        $new->{PDL} = $p;
+        $new->{PDL} = $p->setbadif( $p < 0 );
         return $new;
     }
 
@@ -132,20 +156,12 @@ sub new {
         $levels = $class->_extract_levels($enum);
     }
 
-    # this is faster than Data::Rmap::rmap().
-    state $rmap = sub {
-        my ( $x, $levels ) = @_;
-        is_plain_arrayref($x)
-          ? [ map { __SUB__->( $_, $levels ) } @$x ]
-          : ( $levels->{$x} // -1 );
-    };
-
-    my %levels = do {
-        my $i = 0;
-        map { $_ => $i++; } @$levels;
-    };
-    $enum =
-      $rmap->( ( $enum->$_DOES('PDL') ? $enum->unpdl : $enum ), \%levels );
+    $enum = $enum->$_DOES('PDL') ? $enum->unpdl : dclone($enum);
+    my $i = 0;
+    my %levels = map { $_ => $i++; } @$levels;
+    rmap {
+        $_ = ($levels{$_} // -1); # assign index of level
+    } $enum;
 
     my $self = $class->initialize();
 
@@ -291,7 +307,7 @@ sub _compare_levels {
 # > levels(g$Species) <- levels(g$Species)[c(3, 2, 1)]
 # > iris$Species == g$Species
 # : # outputs a logical vector where only 'versicolor' indices are TRUE
-sub _eq {
+sub equal {
 	my ($self, $other, $d) = @_;
 	# TODO need to look at $d to determine direction
 	if( blessed($other) && $other->isa('PDL::Factor') ) {
@@ -317,7 +333,9 @@ sub _eq {
 	}
 }
 
-sub _ne { !_eq(@_); }
+sub not_equal {
+	return !equal(@_);
+}
 
 
 1;
@@ -334,7 +352,7 @@ PDL::Factor - PDL subclass for keeping categorical data
 
 =head1 VERSION
 
-version 0.0059_01
+version 0.0041
 
 =head1 SYNOPSIS
 
@@ -380,7 +398,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014, 2019-2021 by Zakariyya Mughal, Stephan Loyd.
+This software is copyright (c) 2014, 2019 by Zakariyya Mughal, Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
