@@ -5,10 +5,8 @@ package PDL::Logical;
 use 5.016;
 use warnings;
 
-use PDL::Lite ();   # PDL::Lite is the minimal to get PDL work
 use PDL::Core qw(pdl);
-
-use Ref::Util qw(is_plain_arrayref);
+use Data::Rmap qw(rmap_array);
 use Safe::Isa;
 
 use parent 'PDL';
@@ -26,17 +24,14 @@ sub new {
     if ( $data->$_DOES('PDL') ) {
         $data = !!$data;
     }
-    elsif ( is_plain_arrayref($data) ) {
-
-        # this is faster than Data::Rmap::rmap().
-        state $rmap = sub {
-            my ($x) = @_;
-            is_plain_arrayref($x)
-              ? [ map { __SUB__->($_) } @$x ]
-              : ( $x ? 1 : 0 );
-        };
-
-        $data = pdl( $rmap->($data) );
+    elsif ( ref($data) eq 'ARRAY' ) {
+        my ($data1) = rmap_array {
+            ( ref( $_->[0] ) eq 'ARRAY' )
+              ? [ $_[0]->recurse() ]
+              : [ map { $_ ? 1 : 0 } @$_ ];
+        }
+        $data;
+        $data = pdl($data1);
     }
     else {
         $data = pdl( $data ? 1 : 0 );
@@ -67,7 +62,7 @@ PDL::Logical - PDL subclass for keeping logical data
 
 =head1 VERSION
 
-version 0.0059_01
+version 0.0041
 
 =head1 SYNOPSIS
 
@@ -102,7 +97,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014, 2019-2021 by Zakariyya Mughal, Stephan Loyd.
+This software is copyright (c) 2014, 2019 by Zakariyya Mughal, Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
