@@ -15,20 +15,20 @@ with qw(PDL::Role::Stringifiable);
 requires 'levels';
 
 sub element_stringify_max_width {
-    my ($self, $element) = @_;
-    my @where_levels = @{ $self->{PDL}->uniq->unpdl };
-    my @which_levels = @{ $self->levels }[@where_levels];
-    my @lengths = map { length $_ } @which_levels;
-    List::AllUtils::max( @lengths );
+	my ($self, $element) = @_;
+	my @where_levels = @{ $self->{PDL}->uniq->unpdl };
+	my @which_levels = @{ $self->levels }[@where_levels];
+	my @lengths = map { length $_ } @which_levels;
+	List::AllUtils::max( @lengths );
 }
 
 sub element_stringify {
-    my ($self, $element) = @_;
-    $self->levels->[ $element ];
+	my ($self, $element) = @_;
+	$self->levels->[ $element ];
 }
 
 sub number_of_levels {
-    my ($self) = @_;
+	my ($self) = @_;
     return scalar( @{ $self->levels } );
 }
 
@@ -42,16 +42,12 @@ sub uniq {
 }
 
 around qw(slice dice) => sub {
-    my $orig = shift;
-    my $self = shift;
-    my $class = ref($self);
-
-    my $pdl = $self->{PDL}->$orig(@_);
-    my $ret = bless( { PDL => $pdl }, $class );
-
-    # TODO levels needs to be copied
-    $ret->levels( $self->levels );
-    return $ret;
+	my $orig = shift;
+	my ($self) = @_;
+	my $ret = $orig->(@_);
+	# TODO levels needs to be copied
+	$ret->levels( $self->levels );
+	$ret;
 };
 
 1;
@@ -68,7 +64,7 @@ PDL::Role::Enumerable
 
 =head1 VERSION
 
-version 0.0059_01
+version 0.0041
 
 =head1 AUTHORS
 
@@ -86,7 +82,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014, 2019-2021 by Zakariyya Mughal, Stephan Loyd.
+This software is copyright (c) 2014, 2019 by Zakariyya Mughal, Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
