@@ -15,16 +15,10 @@ sub element_stringify {
 sub string {
 	# TODO
 	my ($self) = @_;
-
-    if ($self->nelem > $PDL::toolongtoprint) {
-        return "TOO LONG TO PRINT";
-    }
-
-    my $ndims = $self->ndims;
-	if( $ndims == 0 ) {
+	if( $self->ndims == 0 ) {
 		return $self->element_stringify( $self->at() );
 	}
-	elsif( $ndims == 1 ) {
+	if( $self->ndims == 1 ) {
 		return $self->string1d;
 	}
 	# TODO string2d, stringNd
@@ -63,7 +57,7 @@ PDL::Role::Stringifiable
 
 =head1 VERSION
 
-version 0.0059_01
+version 0.0041
 
 =head1 AUTHORS
 
@@ -81,7 +75,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014, 2019-2021 by Zakariyya Mughal, Stephan Loyd.
+This software is copyright (c) 2014, 2019 by Zakariyya Mughal, Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
