@@ -5,12 +5,11 @@ package PDL::SV;
 use 5.016;
 use warnings;
 
-use PDL::Lite ();   # PDL::Lite is the minimal to get PDL work
 use PDL::Core qw(pdl);
-use PDL::Primitive qw(which);
-
-use Ref::Util qw(is_plain_arrayref);
+use PDL::Primitive qw(which whichND);
+use Data::Rmap qw(rmap_array);
 use Safe::Isa;
+use Storable qw(dclone);
 use Type::Params;
 use Types::Standard qw(slurpy ArrayRef ConsumerOf Int);
 use List::AllUtils ();
@@ -21,14 +20,14 @@ use Class::Method::Modifiers;
 use Role::Tiny::With;
 with qw(PDL::Role::Stringifiable);
 
-use Devel::OverloadInfo qw(overload_op_info);
+use Devel::OverloadInfo qw(overload_info);
 
 my $overload_info;
 my $super_dotassign;
 
 BEGIN {
-    my $overload_info = overload_op_info('PDL', '.=');
-    $super_dotassign = $overload_info->{code};
+    my $overload_info = overload_info('PDL');
+    $super_dotassign = $overload_info->{'.='}{code};
 }
 
 use overload
@@ -48,31 +47,16 @@ use overload
   '.=' => sub {
     my ( $self, $other, $swap ) = @_;
 
-    if ( $other->$_DOES('PDL::SV') ) {
-        my $internal = $self->_internal;
-        if ($other->dim(0) == 1) {
-            for my $i ( 0 .. $self->dim(0) - 1 ) {
-                my $idx = PDL::Core::at( $self, $i );
-                $internal->[$idx] = $other->at(0);
-            }
-        } else {
-            for my $i ( 0 .. $self->dim(0) - 1 ) {
-                my $idx = PDL::Core::at( $self, $i );
-                $internal->[$idx] = $other->at($i);
-            }
-        }
-        return $self;
-    }
-    elsif ($other->$_DOES('PDL')) {
+    unless ( $other->$_DOES('PDL::SV') ) {
         return $super_dotassign->( $self, $other, $swap );
     }
-    else {  # non-piddle
-        my $internal = $self->_internal;
-        for my $i ( 0 .. $self->dim(0) - 1 ) {
-            my $idx = PDL::Core::at( $self, $i );
-            $internal->[$idx] = $other;
-        }
+
+    my $internal = $self->_internal;
+    for my $i ( 0 .. $other->dim(0) - 1 ) {
+        my $idx = PDL::Core::at( $self, $i );
+        $internal->[$idx] = $other->at($i);
     }
+    return $self;
   },
   fallback => 1;
 
@@ -93,25 +77,22 @@ sub new {
     my ( $class, @args ) = @_;
     my $data = shift @args;    # first arg
 
-    state $rmap = sub {
-        my ( $x ) = @_; 
-        is_plain_arrayref($x)
-          ? [ map { __SUB__->( $_ ) } @$x ]
-          : 0;
-    };  
-
-    my $faked_data = $rmap->($data);
+    my ($faked_data) = rmap_array {
+        ( ref( $_->[0] ) eq 'ARRAY' ) ? [ $_[0]->recurse() ] : [ (0) x @$_ ]
+    }
+    $data;
 
     my $self = $class->initialize();
     my $pdl  = $self->{PDL};
     $pdl .= PDL::Core::indx($faked_data);
     $pdl .= PDL->sequence( $self->dims );
 
+    my $nelem    = $self->nelem;
     if ($self->ndims == 1) {    # for speed 
         $self->_internal($data);
     } else {
         my $internal = $self->_internal;
-        for my $idx ( 0 .. $self->nelem - 1 ) {
+        for my $idx ( 0 .. $nelem - 1 ) {
             my @where = reverse $self->one2nd($idx);
             $internal->[$idx] = $self->_array_get( $data, \@where );
         }
@@ -128,7 +109,7 @@ sub initialize {
 # code modified from <https://metacpan.org/pod/Hash::Path>
 sub _array_get {
     my ( $self, $array, $indices ) = @_;
-
+    return $array unless scalar @$indices;
     my $return_value = $array->[ $indices->[0] ];
     for ( 1 .. $#$indices ) {
         $return_value = $return_value->[ $indices->[$_] ];
@@ -147,14 +128,13 @@ sub _array_set {
     $subarray->[ $indices->[-1] ] = $val;
 }
 
-
 around qw(slice dice) => sub : lvalue {
     my $orig = shift;
     my $self = shift;
-    my $class = ref($self);
 
-    my $pdl = $self->{PDL}->$orig(@_);
-    return bless( { PDL => $pdl, _internal => $self->_internal }, $class );
+    my $new = $self->$orig(@_);
+    $new->_internal( $self->_internal );
+    return $new;
 };
 
 
@@ -181,48 +161,19 @@ sub glue {
     return $new;
 }
 
-
 sub uniq {
-    my ($self) = @_;
+    my $self  = shift;
     my $class = ref($self);
 
-    my @uniq = List::AllUtils::uniq( grep { defined $_ }
-          @{ $self->_effective_internal } );
-    return $class->new( \@uniq );
-}
-
-## Please see file perltidy.ERR
-sub uniqind {
-    my ($self) = @_;
-
-    my $effective_internal = $self->_effective_internal;
-    my @uniqind = List::AllUtils::uniq_by { $effective_internal->[$_] }
-        grep { defined $effective_internal->[$_] }
-            ( 0 .. $#$effective_internal );
-    return pdl( \@uniqind );
-}
-
-
-sub sever {
-    my ($self) = @_;
-
-    $self->_internal( $self->_effective_internal );
-    my $p = PDL->sequence( $self->dims );
-    $p = $p->setbadif( $self->isbad ) if $self->badflag;
-    $self->{PDL} = $p;
-    return $self;
-}
-
-
-sub set {
-    my ($self, @position) = @_;
-
-    my $value = pop @position;
-    my $idx = $self->{PDL}->at(@position);
-    $self->_internal->[$idx] = $value;
-    return $self;
+    my $uniq = [ List::AllUtils::uniq( @{ $self->_internal } ) ];
+    return $class->new($uniq);
 }
 
+#around qw(sever) => sub {
+#	# TODO
+#	# clone the contents of _internal
+#	# renumber the elements
+#};
 
 sub at {
     my $self = shift;
@@ -232,59 +183,52 @@ sub at {
     return $self->_internal->[$idx];
 }
 
-
 sub unpdl {
     my $self = shift;
 
-    if ($self->ndims == 1) {    # shortcut for 1D for performance
-        return [ $self->list ];
-    }
-
-    state $rmap = sub {
-        my ( $x, $f ) = @_;
-        is_plain_arrayref($x)
-          ? [ map { __SUB__->( $_, $f ) } @$x ]
-          : $f->($x);
-    };
-
+    my $data     = $self->{PDL}->unpdl;
     my $internal = $self->_internal;
-    my $f =
-      $self->badflag
-      ? sub { $_ eq 'BAD' ? 'BAD' : $internal->[$_] }
-      : sub { $internal->[$_] };
-    return $rmap->( $self->{PDL}->unpdl, $f );
+    if ($self->ndims == 1) {    # for speed
+        my $f =
+          $self->badflag
+          ? sub { $_ eq 'BAD' ? 'BAD' : $internal->[$_] }
+          : sub { $internal->[$_] };
+        $data = [ map { $f->($_) } @$data ];
+    } else {
+        my $f =
+          $self->badflag
+          ? sub { $_ = ( $_ eq 'BAD' ? 'BAD' : $internal->[$_] ); }
+          : sub { $_ = $internal->[$_] };
+        Data::Rmap::rmap_scalar { $f->($_) } $data;
+    }
+    return $data;
 }
 
-
 sub list {
     my ($self) = @_;
 
+    my @data     = $self->{PDL}->list;
     my $internal = $self->_internal;
-    my @list = do {
-        no warnings 'numeric';
-        map { $internal->[$_] } $self->{PDL}->list;
-    };
     if ($self->badflag) {
-        my @bad_indices = which($self->isbad)->list;
-        @list[@bad_indices] = (('BAD') x @bad_indices);
+        return (map { $_ eq 'BAD' ? 'BAD' : $internal->[$_] } @data);
+    } else {
+        return (map { $internal->[$_] } @data);
     }
-    return @list;
 }
 
-
+#TODO: reimplement to reduce memory usage
 sub copy {
     my ($self) = @_;
 
     my $new = PDL::SV->new( [] );
     $new->{PDL} = PDL->sequence( $self->dims );
-    $new->_internal( $self->_effective_internal );
     if ( $self->badflag ) {
         $new->{PDL} = $new->{PDL}->setbadif( $self->isbad );
     }
+    $new->_internal( [ map { $_ // 'BAD' } @{ $self->_effective_internal } ] );
     return $new;
 }
 
-
 sub inplace {
     my $self = shift;
     $self->{PDL}->inplace(@_);
@@ -300,19 +244,11 @@ sub _call_on_pdl {
     };
 }
 
-
-sub where {
-    my ( $self, $mask ) = @_;
-    return $self->slice( which($mask) );
-}
-
-
 for my $method (qw(isbad isgood ngood nbad)) {
     no strict 'refs';
     *{$method} = _call_on_pdl($method);
 }
 
-
 sub setbadif {
     my $self = shift;
 
@@ -351,19 +287,18 @@ sub match_regexp {
     return $p;
 }
 
+sub where {
+    my ( $self, $mask ) = @_;
+    return $self->slice( which($mask) );
+}
+
 sub _effective_internal {
     my ($self) = @_;
 
     my $internal = $self->_internal;
-    my @indices = $self->{PDL}->list;
-
-    no warnings 'numeric';
-    my @rslt = @$internal[@indices];
-    if ($self->badflag) {
-        my @isbad = which($self->isbad)->list;
-        @rslt[@isbad] = ((undef) x @isbad);
-    }
-    return \@rslt;
+    my $rslt =
+      [ map { $_ eq 'BAD' ? undef : $internal->[$_] } ( $self->{PDL}->list ) ];
+    return $rslt;
 }
 
 sub _compare {
@@ -444,7 +379,7 @@ PDL::SV - PDL subclass for keeping scalar data (like strings)
 
 =head1 VERSION
 
-version 0.0059_01
+version 0.0041
 
 =head1 SYNOPSIS
 
@@ -452,27 +387,7 @@ version 0.0059_01
     
     my $p = PDL::SV->new( [ qw(foo bar) ] );
 
-=head1 DESCRIPTION
-
-This PDL::SV class stores array of scalar values. It can be used for vectors
-of strings.
-
-While this class is a subclass of L<PDL>, its internals are quite different
-from other normal PDL types. So basically what's not documented are not
-guarenteed to work.
-
-=head1 METHODS / BASIC
-
-These methods basically have similar behavior as PDL class's methods of
-same names.
-
-=head2 slice
-
-    slice(...)
-
-=head2 dice
-
-    dice(...)
+=head1 METHODS
 
 =head2 glue
 
@@ -481,83 +396,10 @@ same names.
 Glue two or more PDLs together along an arbitrary dimension.
 For now it only supports 1D PDL::SV piddles, and C<$dim> has to be C<0>.
 
-=head2 uniq
-
-    uniq()
-
-BAD values are not considered unique and are ignored.
-
-=head2 uniqind()
-
-Return the indices of all uniq elements of a piddle.
-
-=head2 sever
-
-    sever()
-
-=head2 set
-
-    set(@position, $value)
-
-=head2 at
-
-    at(@position)
-
-=head2 unpdl
-
-    unpdl()
-
-=head2 list
-
-    list()
-
-=head2 copy
-
-    copy()
-
-=head2 inplace
-
-    inplace()
-
-=head2 where
-
-    where($mask)
-
-=head1 METHODS / BAD VALUE
-
-These methods basically have similar behavior as PDL class's methods of
-same names.
-
-=head2 isbad
-
-    isbad()
-
-=head2 isgood
-
-    isgood()
-
-=head2 ngood
-
-    ngood()
-
-=head2 nbad
-
-    nbad()
-
-=head2 setbadif
-
-    setbadif($mask)
-
 =head2 setbadtoval
 
-    setbadtoval($val)
-
 Cannot be run inplace.
 
-=head1 METHODS / ADDITIONAL
-
-These methods exist not in PDL but only in this class.
-
 =head2 match_regexp
 
     match_regexp($pattern)
@@ -585,7 +427,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014, 2019-2021 by Zakariyya Mughal, Stephan Loyd.
+This software is copyright (c) 2014, 2019 by Zakariyya Mughal, Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
