@@ -2,7 +2,7 @@ package PDL::StringfiableExtension;
 
 use strict;
 use warnings;
-use PDL::Lite ();
+use PDL::Lite;
 use List::AllUtils ();
 
 
@@ -44,7 +44,7 @@ PDL::StringfiableExtension
 
 =head1 VERSION
 
-version 0.0059_01
+version 0.0041
 
 =head1 AUTHORS
 
@@ -62,7 +62,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014, 2019-2021 by Zakariyya Mughal, Stephan Loyd.
+This software is copyright (c) 2014, 2019 by Zakariyya Mughal, Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
