@@ -6,14 +6,14 @@ use 5.010;
 use strict;
 use warnings;
 
-our $VERSION = '0.0059_01'; # TRIAL VERSION
+our $VERSION = '0.0041'; # VERSION
 
 use Safe::Isa;
 use Test2::API qw/context/;
 use Test2::Util::Table qw/table/;
 use Test2::Util::Ref qw/render_ref/;
 
-use parent qw(Exporter::Tiny);
+use parent qw/Exporter/;
 our @EXPORT = qw(dataframe_ok dataframe_is);
 
 
@@ -38,7 +38,8 @@ sub dataframe_is ($$;$@) {
     my ( $got, $exp, $name, @diag ) = @_;
     my $ctx = context();
 
-    local $Data::Frame::TOLERANCE_REL = 1e-8 unless $Data::Frame::TOLERANCE_REL;
+    # TODO: Make this a package variable.
+    local $Data::Frame::TOLERANCE_REL = 1e-9;
 
     unless ( $got->$_DOES('Data::Frame') ) {
         my $gotname = render_ref($got);
@@ -108,7 +109,7 @@ Test2::Tools::DataFrame - Tools for verifying Data::Frame data frames
 
 =head1 VERSION
 
-version 0.0059_01
+version 0.0041
 
 =head1 SYNOPSIS
 
@@ -158,7 +159,7 @@ Stephan Loyd <sloyd@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014, 2019-2021 by Zakariyya Mughal, Stephan Loyd.
+This software is copyright (c) 2014, 2019 by Zakariyya Mughal, Stephan Loyd.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
